#  Copyright 2024 Exactpro (Exactpro Systems Limited)
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

from typing import Union

from th2_data_services.data_source.lwdp.data_source import DataSource as LwdpDataSource
from th2_data_services.data_source.lwdp.struct import http_event_struct, http_message_struct
from th2_data_services.data_source.lwdp.stub_builder import http_event_stub_builder, http_message_stub_builder
from th2_data_services.interfaces import IEventStruct, IMessageStruct, IEventStub, IMessageStub
from th2_data_services.data_source.rdp.source_api.http import API

class DataSource(LwdpDataSource):
    def __init__(
        self,
        url: str,
        chunk_length: int = 65536,
        event_struct: IEventStruct = http_event_struct,
        message_struct: IMessageStruct = http_message_struct,
        event_stub_builder: IEventStub = http_event_stub_builder,
        message_stub_builder: IMessageStub = http_message_stub_builder,
        check_connect_timeout: Union[int, float] = 5
    ):
        super().__init__(url, chunk_length, event_struct, message_struct, event_stub_builder, message_stub_builder, check_connect_timeout)
        self._provider_api = API(url, chunk_length)


HTTPDataSource = DataSource



#  Copyright 2024 Exactpro (Exactpro Systems Limited)
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

from th2_data_services.data_source.lwdp.source_api import API as LwdpAPI


class API(LwdpAPI):
    def __encode_url(self, url: str) -> str:
        return self._API__encode_url(url)

    def get_url_get_books(self) -> str:
        """REST-API `books` call returns a list of books in cradleAPI."""
        return self.__encode_url(f"{self._url}/bookIds")

    def get_url_get_message_aliases(self, book_id: str, start_timestamp: int = None, end_timestamp: int = None,
                                    chunked_size: int = None) -> str:
        """REST-API `messageStreams?bookId={bookID}` call returns a list of aliases in cradleAPI."""
        url = f"{self._url}/messageStreams?bookId={book_id}"
        return self.__encode_url(url)

    def get_url_get_scopes(self, book_id: str, start_timestamp: int = None, end_timestamp: int = None,
                           chunked_size: int = None) -> str:
        """REST-API `scopeIds?bookId={bookID}` call returns a list of scopes in cradleAPI."""
        url = f"{self._url}/scopeIds?bookId={book_id}"
        return self.__encode_url(url)


HTTPAPI = API



#  Copyright 2024 Exactpro (Exactpro Systems Limited)
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

from typing import List

from th2_data_services.data_source.lwdp.commands.http import GetBooks as LwdpGetBooks
from th2_data_services.data_source.lwdp.interfaces.command import IHTTPCommand
from th2_data_services.data_source.rdp.data_source.http import DataSource
from th2_data_services.data_source.rdp.source_api.http import API


class GetBooks(LwdpGetBooks):
    def handle(self, data_source: DataSource) -> List[str]:
        return list(map(lambda book: book['name'], super().handle(data_source)))


class GetMessageAliases(IHTTPCommand):
    def __init__(self, book_id: str):
        """GetMessageAliases constructor."""
        super().__init__()
        self._book_id = book_id

    def handle(self, data_source: DataSource) -> List[str]:
        api: API = data_source.source_api
        url = api.get_url_get_message_aliases(self._book_id)
        return api.execute_request(url).json()


class GetEventScopes(IHTTPCommand):
    def __init__(self, book_id: str):
        """GetEventScopes constructor."""
        super().__init__()
        self._book_id = book_id

    def handle(self, data_source: DataSource) -> List[str]:
        api: API = data_source.source_api
        url = api.get_url_get_scopes(self._book_id)
        return api.execute_request(url).json()
